import {
  ExceptionFilter,
  Catch,
  ArgumentsHost,
  HttpException,
  HttpStatus,
} from '@nestjs/common';
import { ApolloError } from 'apollo-server-errors';

@Catch(HttpException)
export class HttpExceptionFilter implements ExceptionFilter {
  catch(exception: HttpException, host: ArgumentsHost) {
    switch (host.getType<'http' | 'graphql' | string>()) {
      case 'graphql': {
        const message = exception.message;
        const extensions: Record<string, any> = {};
        const err = exception.getResponse();
        if (typeof err === 'string') {
          extensions.message = err;
        } else {
          Object.assign(extensions, (err as any).extension);
          extensions.message = (err as any).message;
        }
        return new ApolloError(
          message,
          exception.getStatus().toString(),
          extensions,
        );
      }
      case 'http': {
        const ctx = host.switchToHttp();
        const response = ctx.getResponse();
        const request = ctx.getRequest();

        const status =
          exception instanceof HttpException
            ? exception.getStatus()
            : HttpStatus.INTERNAL_SERVER_ERROR;

        response.status(status).json({
          statusCode: status,
          message: exception.message,
          timestamp: new Date().toISOString(),
          path: request.url,
        });
      }
      default:
        throw exception;
    }
  }
}
